=== modified file 'lib/canonical/launchpad/database/archive.py'
--- lib/canonical/launchpad/database/archive.py	2008-03-21 17:03:19 +0000
+++ lib/canonical/launchpad/database/archive.py	2008-03-27 16:29:13 +0000
@@ -28,6 +28,7 @@
 from canonical.launchpad.database.distroseriespackagecache import (
     DistroSeriesPackageCache)
 from canonical.launchpad.database.librarian import LibraryFileContent
+from canonical.launchpad.database.publishedpackage import PublishedPackage
 from canonical.launchpad.database.publishing import (
     SourcePackagePublishingHistory, BinaryPackagePublishingHistory)
 from canonical.launchpad.interfaces import (
@@ -114,6 +115,17 @@
         return dependencies
 
     @property
+    def expanded_archive_dependencies(self):
+        """See `IArchive`."""
+        archives = []
+        if self.purpose == ArchivePurpose.PPA:
+            archives.append(self.distribution.main_archive)
+        archives.append(self)
+        archives.extend(
+            [archive_dep.dependency for archive_dep in self.dependencies])
+        return archives
+
+    @property
     def archive_url(self):
         """See `IArchive`."""
         archive_postfixes = {
@@ -535,6 +547,21 @@
         self.sources_cached = sources_cached.count()
         self.binaries_cached = binaries_cached.count()
 
+    def findDepCandidateByName(self, distroarchseries, name):
+        """See `IArchive`."""
+        archives = [
+            archive.id for archive in self.expanded_archive_dependencies]
+
+        query = """
+            binarypackagename = %s AND
+            distroarchseries = %s AND
+            archive IN %s AND
+            packagepublishingstatus = %s
+        """ % sqlvalues(name, distroarchseries, archives,
+                        PackagePublishingStatus.PUBLISHED)
+
+        return PublishedPackage.selectFirst(query, orderBy=['-id'])
+
     def getArchiveDependency(self, dependency):
         """See `IArchive`."""
         return ArchiveDependency.selectOneBy(

=== modified file 'lib/canonical/launchpad/database/build.py'
--- lib/canonical/launchpad/database/build.py	2008-03-17 11:45:04 +0000
+++ lib/canonical/launchpad/database/build.py	2008-03-27 17:15:49 +0000
@@ -297,7 +297,9 @@
         satisfied and if it is reachable in the build context.
         """
         name, version, relation = self._parseDependencyToken(token)
-        dep_candidate = self.distroarchseries.findDepCandidateByName(name)
+
+        dep_candidate = self.archive.findDepCandidateByName(
+            self.distroarchseries, name)
 
         if not dep_candidate:
             return False
@@ -306,7 +308,14 @@
             dep_candidate.binarypackageversion, version, relation):
             return False
 
-        if dep_candidate.component not in self.ogre_components:
+        # Only PRIMARY archive build dependencies should be restricted
+        # to the ogre_components. Both PARTNER and PPA can reach
+        # dependencies from all components in the PRIMARY archive.
+        # Moreover, PARTNER and PPA component domain is single, i.e,
+        # PARTNER only contains packages in 'partner' component and PPAs
+        # only contains packages in 'main' component.
+        if (self.archive.purpose == ArchivePurpose.PRIMARY and
+            dep_candidate.component not in self.ogre_components):
             return False
 
         return True

=== modified file 'lib/canonical/launchpad/database/distroarchseries.py'
--- lib/canonical/launchpad/database/distroarchseries.py	2008-02-15 18:57:21 +0000
+++ lib/canonical/launchpad/database/distroarchseries.py	2008-03-27 15:16:26 +0000
@@ -29,7 +29,6 @@
 from canonical.launchpad.validators.person import public_person_validator
 from canonical.launchpad.database.publishing import (
     BinaryPackagePublishingHistory)
-from canonical.launchpad.database.publishedpackage import PublishedPackage
 from canonical.launchpad.database.processor import Processor
 from canonical.launchpad.database.binarypackagerelease import (
     BinaryPackageRelease)
@@ -231,13 +230,6 @@
 
         return shortlist(published)
 
-    def findDepCandidateByName(self, name):
-        """See IPublishedSet."""
-        return PublishedPackage.selectFirstBy(
-            binarypackagename=name, distroarchseries=self,
-            packagepublishingstatus=PackagePublishingStatus.PUBLISHED,
-            orderBy=['-id'])
-
     def getPendingPublications(self, archive, pocket, is_careful):
         """See ICanPublishPackages."""
         queries = [

=== modified file 'lib/canonical/launchpad/doc/archive.txt'
--- lib/canonical/launchpad/doc/archive.txt	2008-03-19 13:35:34 +0000
+++ lib/canonical/launchpad/doc/archive.txt	2008-04-02 14:38:46 +0000
@@ -702,6 +702,13 @@
     >>> pubconfig.componentsForSeries('breezy-autotest')
     ['main', 'restricted', 'universe', 'multiverse']
 
+Let's restore Celso's PPA state, attaching it to ubuntu distribution
+again.
+
+    >>> ubuntu = getUtility(IDistributionSet)['ubuntu']
+    >>> naked_cprov_archive.distribution = ubuntu
+    >>> flush_database_updates()
+
 
 == Archive dependencies ==
 
@@ -810,6 +817,80 @@
     >>> print_dependencies(no_priv.archive)
     No dependencies recorded.
 
+Since the 'dependency' property only returns explicitly recorded
+archive dependencies and we usually need the expanded list when
+inspecting the build domain we have also implemented the
+'expanded_archive_dependencies' property. It returns implicit
+dependencies, including the archive itself and the PRIMARY archive
+dependency for PPAs.
+
+    >>> def print_expanded_archive_dependencies(archive):
+    ...     for dep in archive.expanded_archive_dependencies:
+    ...         print dep.title
+
+    >>> print_expanded_archive_dependencies(ubuntu.main_archive)
+    Primary Archive for Ubuntu Linux
+
+    >>> print_expanded_archive_dependencies(cprov.archive)
+    Primary Archive for Ubuntu Linux
+    PPA for Celso Providelo
+    PPA for Mark Shuttleworth
+
+
+== Find binary package dependency candidates ==
+
+Archive allows a lookup on a single binary package dependency
+candidate by its name, via the `PublishedPackage` table:
+
+    >>> warty_i386 = warty['i386']
+
+    >>> candidate = ubuntu.main_archive.findDepCandidateByName(
+    ...     warty_i386, "pmount")
+    >>> print candidate.binarypackagename
+    pmount
+
+    >>> candidate = cprov.archive.findDepCandidateByName(
+    ...     warty_i386, "pmount")
+    >>> print candidate.binarypackagename
+    pmount
+
+Since 'python2.4' isn't available in our sampledata (not even
+published), None is returned:
+
+    >>> print ubuntu.main_archive.findDepCandidateByName(
+    ...     warty_i386, "python2.4")
+    None
+
+    >>> print cprov.archive.findDepCandidateByName(
+    ...     warty_i386, "python2.4")
+    None
+
+This method is aware of the archive dependency tree. So, even when a
+package is not published on the context PPA but is available somewhere
+in the archive dependency domain it will be found.
+
+'at' binary package is not present in Celso's PPA.
+
+    >>> cprov_archive.getAllPublishedBinaries(name='at').count()
+    0
+
+But it is available in PRIMARY ubuntu archive.
+
+    >>> primary_candidate = ubuntu.main_archive.findDepCandidateByName(
+    ...     warty_i386, "at")
+    >>> primary_candidate is not None
+    True
+
+Then a lookup on Celso's PPA will find it.
+
+    >>> ppa_candidate = cprov.archive.findDepCandidateByName(
+    ...     warty_i386, "at")
+    >>> ppa_candidate is not None
+    True
+
+    >>> primary_candidate == ppa_candidate
+    True
+
 
 == IArchiveSet Utility ==
 
@@ -892,11 +973,11 @@
 are published to a secure location.
 
     >>> naked_cprov_archive.archive_url
-    u'http://ppa.launchpad.dev/cprov/ubuntutest'
+    u'http://ppa.launchpad.dev/cprov/ubuntu'
 
     >>> naked_cprov_archive.private = True
     >>> naked_cprov_archive.archive_url
-    u'http://private.ppa.launchpad.dev/cprov/ubuntutest'
+    u'http://private.ppa.launchpad.dev/cprov/ubuntu'
 
 The publisher config for PARTNER contains only 'partner' in its
 components.  This prevents non-partner being published in the partner
@@ -928,7 +1009,6 @@
 `PackageLocation` to provide a homogeneous way to refer to a Location
 (archive, distribution, distroseries, pocket).
 
-    >>> ubuntu = getUtility(IDistributionSet)['ubuntu']
     >>> cprov_archive == archive_set.getPPAByDistributionAndOwnerName(
     ...     ubuntu, 'cprov')
     True

=== modified file 'lib/canonical/launchpad/doc/build.txt'
--- lib/canonical/launchpad/doc/build.txt	2008-03-21 17:03:19 +0000
+++ lib/canonical/launchpad/doc/build.txt	2008-03-27 17:15:49 +0000
@@ -795,6 +795,37 @@
     >>> print depwait_build.dependencies
     mozilla-firefox
 
+'pmount' dependency is also satisfied in the Celso's PPA context,
+even when it is published in a component not allowed in its current
+component domain ('ogre_components'). That's because PPAs implicitly
+depend on all components of its distribution PRIMARY archive.
+
+    >>> depwait_build.dependencies = 'biscuit, pmount'
+
+    >>> login('foo.bar@canonical.com')
+    >>> universe_component = getUtility(IComponentSet)['universe']
+    >>> pmount_pub.secure_record.component = universe_component
+    >>> removeSecurityProxy(depwait_build).archive = cprov.archive
+    >>> flush_database_caches()
+    >>> login(ANONYMOUS)
+
+    >>> print depwait_build.ogre_components
+    ['main']
+
+    >>> print pmount_pub.component.name
+    universe
+
+    >>> depwait_build.updateDependencies()
+    >>> print depwait_build.dependencies
+    biscuit
+
+Restore depwait_build previous state.
+
+    >>> login('foo.bar@canonical.com')
+    >>> pmount_pub.secure_record.component = main_component
+    >>> removeSecurityProxy(depwait_build).archive = hoary.main_archive
+    >>> flush_database_caches()
+    >>> login(ANONYMOUS)
 
 === Retrying DEPWAIT builds ===
 

=== modified file 'lib/canonical/launchpad/doc/distroarchseries.txt'
--- lib/canonical/launchpad/doc/distroarchseries.txt	2007-12-19 17:39:45 +0000
+++ lib/canonical/launchpad/doc/distroarchseries.txt	2008-04-02 14:38:46 +0000
@@ -3,7 +3,6 @@
    >>> from canonical.launchpad.webapp.testing import verifyObject
 
    >>> from canonical.launchpad.interfaces import (
-   ...    IDistroArchSeriesBinaryPackageRelease,
    ...    IDistroArchSeriesBinaryPackage,
    ...    IBinaryPackagePublishingHistory,
    ...    ISectionSet)
@@ -32,19 +31,6 @@
 
 See more information about getReleasePackages below.
 
-DistroArchSeries class has a built-in method to return a single
-dependency package candidate by its name as a PublishedPackage
-instance:
-
-   >>> hoary_i386.findDepCandidateByName("pmount").binarypackagename
-   u'pmount'
-
-Since 'at' isn't available in our sampledata (not even published),
-None is returned:
-
-   >>> hoary_i386.findDepCandidateByName("at") is None
-   True
-
 Check the behavior of the provided search method, which returns a
 list of IDARBPR instances containing the matching packages.
 
@@ -144,8 +130,7 @@
 ubuntu/breezy-autotest/i386, which is empty:
 
     >>> from canonical.launchpad.interfaces import (
-    ...    IDistributionSet, IPersonSet, PackagePublishingPocket,
-    ...    PackagePublishingStatus)
+    ...    IDistributionSet, IPersonSet, PackagePublishingPocket)
     >>> from canonical.launchpad.database.publishing import (
     ...     SecureBinaryPackagePublishingHistory)
 

=== modified file 'lib/canonical/launchpad/interfaces/archive.py'
--- lib/canonical/launchpad/interfaces/archive.py	2008-03-21 17:03:19 +0000
+++ lib/canonical/launchpad/interfaces/archive.py	2008-03-27 16:29:13 +0000
@@ -99,6 +99,10 @@
         "Archive dependencies recorded for this archive and ordered by owner "
         "displayname.")
 
+    expanded_archive_dependencies = Attribute(
+        "The expanded list of archive dependencies. It includes the implicit "
+        "PRIMARY archive dependency for PPAs.")
+
     archive_url = Attribute("External archive URL.")
 
     title = Attribute("Archive Title.")
@@ -210,6 +214,13 @@
         Person table indexes while searching.
         """
 
+    def findDepCandidateByName(distroarchseries, name):
+        """Return the last published binarypackage by given name.
+
+        Return the PublishedPackage record by binarypackagename or None if
+        not found.
+        """
+
     def getArchiveDependency(dependency):
         """Return the `IArchiveDependency` object for the given dependency.
 

=== modified file 'lib/canonical/launchpad/interfaces/distroarchseries.py'
--- lib/canonical/launchpad/interfaces/distroarchseries.py	2007-12-15 00:33:46 +0000
+++ lib/canonical/launchpad/interfaces/distroarchseries.py	2008-03-27 15:16:26 +0000
@@ -113,13 +113,6 @@
         this distro arch series.
         """
 
-    def findDepCandidateByName(name):
-        """Return the last published binarypackage by given name.
-
-        Return the PublishedPackage record by binarypackagename or None if
-        not found.
-        """
-
 
 class IDistroArchSeriesSet(Interface):
     """Interface for DistroArchSeriesSet"""

